@@ -20,7 +20,7 @@ from __future__ import absolute_import
 
 __author__ = "czerwin@scalyr.com"
 
-if False:
+if False:  # NOSONAR
     from typing import List
     from typing import Type
 
