@@ -22,7 +22,7 @@ from __future__ import print_function
 
 import re
 
-if False:
+if False:  # NOSONAR
     from typing import Union
     from typing import Tuple
     from typing import Callable
@@ -525,7 +525,7 @@ def md5_hexdigest(data):
     encoded_data = data.encode("utf-8")
 
     if not new_md5:
-        m = md5.new()
+        m = md5.new()  # nosec
     else:
         m = md5()
     m.update(encoded_data)
@@ -623,7 +623,7 @@ def get_pid_tid():
             six.text_type(os.getpid()),
             six.text_type(six.moves._thread.get_ident()),
         )
-    except:
+    except Exception:
         return "(pid=%s) (tid=Unknown)" % (six.text_type(os.getpid()))
 
 
@@ -634,7 +634,7 @@ def is_list_of_strings(vals):
         for val in vals:
             if not isinstance(val, six.string_types):
                 return False
-    except:
+    except Exception:
         # vals is not enumerable
         return False
 
@@ -1324,6 +1324,8 @@ class RateLimiter(object):
         @param num_bytes: The number of bytes to consume from the rate limit.
         @param current_time: If not none, the value to use as the current time, expressed in seconds past epoch. This
             is used in testing.
+
+        @return: Returns the amount of time this method slept.
         """
         if self.__bucket_fill_rate <= 0.0:
             raise ValueError(
@@ -1334,6 +1336,8 @@ class RateLimiter(object):
             time.sleep(time_to_sleep)
         self.__bucket_contents -= num_bytes
 
+        return time_to_sleep
+
     def _get_time_to_sleep(self, num_bytes, current_time=None):
         """Returns the amount of time in seconds we would need to sleep to have enough capacity to charge num_bytes
         without overflowing. num_bytes values greater than the bucket size are allowed and will simulate overfilling
